@@ -37,36 +37,46 @@
 -- The simplest use is to generate a vector of uniformly distributed values:
 --
 -- @
---   vs <- withSystemRandom (uniformVector 100)
+--   vs \<- 'withSystemRandom' . 'asGenST' $ \\gen -> 'uniformVector' gen 100
 -- @
 --
--- These values can be of any type which is an instance of the class 'Variate'.
+-- These values can be of any type which is an instance of the class
+-- 'Variate'.
 --
--- To generate random values on demand, first 'create' a random number generator.
+-- To generate random values on demand, first 'create' a random number
+-- generator.
 --
 -- @
---   gen <- create
+--   gen <- 'create'
 -- @
 --
--- Keep this generator and use it wherever random values are required. Get a random
--- value using 'uniform' or 'uniformR':
+-- Hold onto this generator and use it wherever random values are
+-- required (creating a new generator is expensive compared to
+-- generating a random number, so you don't want to throw them
+-- away). Get a random value using 'uniform' or 'uniformR':
 --
 -- @
---   v <- uniform gen
+--   v <- 'uniform' gen
 -- @
 --
 -- @
---   v <- uniformR (1, 52) gen
+--   v <- 'uniformR' (1, 52) gen
 -- @
 module System.Random.MWC
     (
     -- * Gen: Pseudo-Random Number Generators
       Gen
-    , GenIO
-    , GenST
     , create
     , initialize
     , withSystemRandom
+    , createSystemRandom
+
+    -- ** Type helpers
+    -- $typehelp
+    , GenIO
+    , GenST
+    , asGenIO
+    , asGenST
 
     -- * Variates: uniformly distributed values
     , Variate(..)
@@ -87,7 +97,6 @@ module System.Random.MWC
 #include "MachDeps.h"
 #endif
 
-import Control.Exception       (IOException, catch)
 import Control.Monad           (ap, liftM, unless)
 import Control.Monad.Primitive (PrimMonad, PrimState, unsafePrimToIO)
 import Control.Monad.ST        (ST)
@@ -101,13 +110,13 @@ import Data.Vector.Generic     (Vector)
 import Data.Word               (Word, Word8, Word16, Word32, Word64)
 import Foreign.Marshal.Alloc   (allocaBytes)
 import Foreign.Marshal.Array   (peekArray)
-import Prelude hiding (catch)
 import qualified Data.Vector.Generic         as G
 import qualified Data.Vector.Unboxed         as I
 import qualified Data.Vector.Unboxed.Mutable as M
 import System.CPUTime   (cpuTimePrecision, getCPUTime)
 import System.IO        (IOMode(..), hGetBuf, hPutStrLn, stderr, withBinaryFile)
 import System.IO.Unsafe (unsafePerformIO)
+import qualified Control.Exception as E
 
 
 
@@ -283,7 +292,7 @@ wordToFloat x      = (fromIntegral i * m_inv_32) + 0.5 + m_inv_33
 
 wordsToDouble :: Word32 -> Word32 -> Double
 wordsToDouble x y  = (fromIntegral u * m_inv_32 + (0.5 + m_inv_53) +
-                     fromIntegral (v .&. 0xFFFFF) * m_inv_52) 
+                     fromIntegral (v .&. 0xFFFFF) * m_inv_52)
     where m_inv_52 = 2.220446049250313080847263336181640625e-16
           m_inv_53 = 1.1102230246251565404236316680908203125e-16
           m_inv_32 = 2.3283064365386962890625e-10
@@ -294,12 +303,20 @@ wordsToDouble x y  = (fromIntegral u * m_inv_32 + (0.5 + m_inv_53) +
 -- | State of the pseudo-random number generator.
 newtype Gen s = Gen (M.MVector s Word32)
 
--- | A shorter name for PRNG state in the IO monad.
+-- | A shorter name for PRNG state in the 'IO' monad.
 type GenIO = Gen (PrimState IO)
 
--- | A shorter name for PRNG state in the ST monad.
+-- | A shorter name for PRNG state in the 'ST' monad.
 type GenST s = Gen (PrimState (ST s))
 
+-- | Constrain the type of an action to run in the 'IO' monad.
+asGenIO :: (GenIO -> IO a) -> (GenIO -> IO a)
+asGenIO = id
+
+-- | Constrain the type of an action to run in the 'ST' monad.
+asGenST :: (GenST s -> ST s a) -> (GenST s -> ST s a)
+asGenST = id
+
 ioff, coff :: Int
 ioff = 256
 coff = 257
@@ -318,7 +335,7 @@ create = initialize defaultSeed
 --
 -- > initialize (singleton 42)
 --
--- > initialize (toList [4, 8, 15, 16, 23, 42])
+-- > initialize (fromList [4, 8, 15, 16, 23, 42])
 --
 -- If a seed contains fewer than 256 elements, it is first used
 -- verbatim, then its elements are 'xor'ed against elements of the
@@ -353,11 +370,11 @@ initialize seed = do
                     | otherwise = G.unsafeIndex seed i
         fini = G.length seed
 {-# INLINE initialize #-}
-                               
+
 -- | An immutable snapshot of the state of a 'Gen'.
-newtype Seed = Seed { 
+newtype Seed = Seed {
     -- | Convert seed into vector.
-    fromSeed :: I.Vector Word32 
+    fromSeed :: I.Vector Word32
     }
     deriving (Eq, Show, Typeable)
 
@@ -389,7 +406,7 @@ acquireSeedTime = do
   let n    = fromIntegral (numerator t) :: Word64
   return [fromIntegral c, fromIntegral n, fromIntegral (n `shiftR` 32)]
 
--- Aquire seed from /dev/urandom
+-- | Acquire seed from /dev/urandom
 acquireSeedSystem :: IO [Word32]
 acquireSeedSystem = do
   let nbytes = 1024
@@ -398,14 +415,14 @@ acquireSeedSystem = do
     nread <- withBinaryFile random ReadMode $
                \h -> hGetBuf h buf nbytes
     peekArray (nread `div` 4) buf
-  
+
 -- | Seed a PRNG with data from the system's fast source of
--- pseudo-random numbers (\"\/dev\/urandom\" on Unix-like systems),
+-- pseudo-random numbers (\"@\/dev\/urandom@\" on Unix-like systems),
 -- then run the given action.
 --
--- This is a heavyweight function, intended to be called only
--- occasionally (e.g. once per thread).  You should use the `Gen` it
--- creates to generate many random numbers.
+-- This is a somewhat expensive function, and is intended to be called
+-- only occasionally (e.g. once per thread).  You should use the `Gen`
+-- it creates to generate many random numbers.
 --
 -- /Note/: on Windows, this code does not yet use the native
 -- Cryptographic API as a source of random numbers (it uses the system
@@ -413,7 +430,7 @@ acquireSeedSystem = do
 -- highly independent.
 withSystemRandom :: PrimMonad m => (Gen (PrimState m) -> m a) -> IO a
 withSystemRandom act = do
-  seed <- acquireSeedSystem `catch` \(_::IOException) -> do
+  seed <- acquireSeedSystem `E.catch` \(_::E.IOException) -> do
     seen <- atomicModifyIORef warned ((,) True)
     unless seen $ do
       hPutStrLn stderr ("Warning: Couldn't open /dev/urandom")
@@ -425,6 +442,11 @@ withSystemRandom act = do
     warned = unsafePerformIO $ newIORef False
     {-# NOINLINE warned #-}
 
+-- | Seed a PRNG with data from the system's fast source of pseudo-random
+-- numbers. All the caveats of 'withSystemRandom' apply here as well.
+createSystemRandom :: IO GenIO
+createSystemRandom = withSystemRandom (return :: GenIO -> IO GenIO)
+
 -- | Compute the next index into the state pool.  This is simply
 -- addition modulo 256.
 nextIndex :: Integral a => a -> Int
@@ -491,13 +513,31 @@ type instance Unsigned Int8  = Word8
 type instance Unsigned Int16 = Word16
 type instance Unsigned Int32 = Word32
 type instance Unsigned Int64 = Word64
-type instance Unsigned Int   = Word
 
 type instance Unsigned Word8  = Word8
 type instance Unsigned Word16 = Word16
 type instance Unsigned Word32 = Word32
 type instance Unsigned Word64 = Word64
-type instance Unsigned Word   = Word
+
+-- This is workaround for bug #25.
+--
+-- GHC-7.6 has a bug (#8072) which results in calculation of wrong
+-- number of buckets in function `uniformRange'. Consequently uniformR
+-- generates values in wrong range.
+--
+-- Bug only affects 32-bit systems and Int/Word data types. Word32
+-- works just fine. So we set Word32 as unsigned counterpart for Int
+-- and Word on 32-bit systems. It's done only for GHC-7.6 because
+-- other versions are unaffected by the bug and we expect that GHC may
+-- optimise code which uses Word better.
+#if (WORD_SIZE_IN_BITS < 64) && (__GLASGOW_HASKELL__ == 706)
+type instance Unsigned Int   = Word32
+type instance Unsigned Word  = Word32
+#else
+type instance Unsigned Int   = Word
+type instance Unsigned Word  = Word
+#endif
+
 
 -- Subtract two numbers under assumption that x>=y and store result in
 -- unsigned data type of same size
@@ -597,7 +637,48 @@ defaultSeed = I.fromList [
 --   /Communications of the ACM/ 46(5):90&#8211;93.
 --   <http://doi.acm.org/10.1145/769800.769827>
 --
--- * Thomas, D.B.; Leong, P.G.W.; Luk, W.; Villasenor, J.D.
---   (2007). Gaussian random number generators.
---   /ACM Computing Surveys/ 39(4).
---   <http://www.cse.cuhk.edu.hk/~phwl/mt/public/archives/papers/grng_acmcs07.pdf>
+-- * Doornik, J.A. (2007) Conversion of high-period random numbers to
+--   floating point.
+--   /ACM Transactions on Modeling and Computer Simulation/ 17(1).
+--   <http://www.doornik.com/research/randomdouble.pdf>
+
+
+-- $typehelp
+--
+-- The functions in this package are deliberately written for
+-- flexibility, and will run in both the 'IO' and 'ST' monads.
+--
+-- This can defeat the compiler's ability to infer a principal type in
+-- simple (and common) cases.  For instance, we would like the
+-- following to work cleanly:
+--
+-- > import System.Random.MWC
+-- > import Data.Vector.Unboxed
+-- >
+-- > main = do
+-- >   v <- withSystemRandom $ \gen -> uniformVector gen 20
+-- >   print (v :: Vector Int)
+--
+-- Unfortunately, the compiler cannot tell what monad 'uniformVector'
+-- should execute in.  The \"fix\" of adding explicit type annotations
+-- is not pretty:
+--
+-- > {-# LANGUAGE ScopedTypeVariables #-}
+-- >
+-- > import Control.Monad.ST
+-- >
+-- > main = do
+-- >   vs <- withSystemRandom $
+-- >         \(gen::GenST s) -> uniformVector gen 20 :: ST s (Vector Int)
+-- >   print vs
+--
+-- As a more readable alternative, this library provides 'asGenST' and
+-- 'asGenIO' to constrain the types appropriately.  We can get rid of
+-- the explicit type annotations as follows:
+--
+-- > main = do
+-- >   vs <- withSystemRandom . asGenST $ \gen -> uniformVector gen 20
+-- >   print (vs :: Vector Int)
+--
+-- This is almost as compact as the original code that the compiler
+-- rejected.
